class TransformationWrapper(object):
    def __init__(self, func, **kwargs):
        # Store the decorated function
        self.func = func
        self.counter = 0

        # Extract the name of the transformation from the given kwargs
        assert "name" in kwargs
        self.name = kwargs.pop("name")

    def write_trafo(self, expr, before):
        from dune.perftool.options import get_option
        if get_option("print_transformations", False):
            import os
            dir = get_option("print_transformations_dir", os.getcwd())
            filename = "trafo_{}_{}_{}.dot".format(self.name, str(self.counter).zfill(4), "in" if before else "out")
            filename = os.join(dir, filename)
            with open(filename,'w') as out:
                from ufl.formatting import ufl2dot
                out.write(ufl2dot(expr))
            if not before:
                self._counter = self._counter + 1

    def __call__(self, expr, *args, **kwargs):
        # We assume that the first argument to any transformation is the expression
        from ufl.classes import Expr
        assert isinstance(expr, Expr)

        # Maybe output the input expression!
        from dune.perftool.options import get_option
        self.write_trafo(expr, True)

        # Call the original function
        ret = self.func(expr, *args, **kwargs)

        # We do also assume that the transformation returns an ufl expression
        assert isinstance(ret, Expr)

        # Maybe output the returned expression
        self.write_trafo(ret, False)

        # return the result
        return ret


def ufl_transformation(**kwargs):
    """ A decorator for ufl transformations. It allows us to output the
    result if needed. """
    return lambda f: TransformationWrapper(f, **kwargs)
